/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
import { assert } from "../util/assert";
import { isU32 } from "../util/number";
import { isObject } from "../util/data-validation";
/**
 * Minimal implementation of Concatenable.
 * This enables to model a number of elements without their availability.
 *
 * For instance ConcatLength(3) can represent "abc", [1, 2, 3], or any
 * 3-elements list.
 */
export class ConcatLength {
    /**
     * @param length {@link Concat#length}
     */
    constructor(length) {
        this.length = length;
        assert(() => isU32(length), "length ∈ u32");
    }
    /**
     * @param x candidate
     * @return object from `x', or undefined if `x' is not valid.
     */
    static fromPlain(x) {
        if (isObject(x) && isU32(x.length)) {
            return new ConcatLength(x.length);
        }
        return undefined;
    }
    // Access
    /** @override */
    slice(lower, excludedUpper) {
        assert(() => isU32(lower), "lower ∈ u32");
        assert(() => lower < this.length, "lower < this.length");
        assert(() => isU32(excludedUpper), "excludedUpper ∈ u32");
        assert(() => excludedUpper <= this.length, "excludedUpper <= this.length");
        assert(() => lower <= excludedUpper, "lower <= excludedUpper");
        return new ConcatLength(excludedUpper - lower);
    }
    /** @override */
    concat(other) {
        assert(() => isU32(this.length + other.length), "valid concatenation");
        return new ConcatLength(this.length + other.length);
    }
}
//# sourceMappingURL=concat.js.map