/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
import { assert } from "../util/assert";
import { isU32 } from "../util/number";
/**
 * Model an insertion in a single-user list.
 */
export class Ins {
    /**
     * @param index Insertion index.
     * @param content Inserted elements.
     */
    constructor(index, content) {
        this.index = index;
        this.content = content;
        assert(() => isU32(index), "index ∈ u32");
        assert(() => content.length > 0, "length ∈ u32");
        this.index = index;
        this.content = content;
    }
    // Access
    /**
     * Number of inserted elements.
     */
    get length() {
        return this.content.length;
    }
    endIndex() {
        return this.index + this.length;
    }
}
/**
 * Model a removal in a single-user list.
 */
export class Del {
    /**
     * @param index Removal index.
     * @param items Number of removed elements.
     */
    constructor(index, length) {
        this.index = index;
        this.length = length;
        assert(() => isU32(index), "index ∈ u32");
        assert(() => isU32(length), "length ∈ u32");
        assert(() => length > 0, "length > 0");
    }
    endIndex() {
        return this.index + this.length;
    }
}
//# sourceMappingURL=local-operation.js.map