/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
import { hashCodeOf } from "../util/number";
/**
 * List which can only be remotely updated using operations.
 * An operation must be delivered exactly once.
 * An insertion must be delivered before a removal that depends on.
 */
export class OpReplicatedList {
    /**
     * @param prefix
     * @return Concatenated version prefixed by `prefix`.
     */
    concatenated(prefix) {
        return this.reduceBlock((acc, b) => acc.concat(b.content), prefix);
    }
    /**
     * Non-cryptographic way to approximate object identity.
     * Do not take the blocks' content into account.
     */
    structuralHashCode() {
        return this.reduceBlock((acc, b) => hashCodeOf([acc, b.structuralHashCode()]), 0);
    }
    applyOp(delta) {
        if (delta.isLengthBlock()) {
            return this.remove(delta);
        }
        else {
            return this.insert(delta);
        }
    }
}
//# sourceMappingURL=op-replicated-list.js.map