/*
    Copyright (C) 2019  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
import { OpLinkedList, EditableOpLinkedList } from "./op-linked-list";
import { assert } from "../../util/assert";
import { DeltaReplicatedList, DeltaEditableReplicatedList, } from "../../core/delta-replicated-list";
export function opList() {
    return OpLinkedList.empty();
}
/**
 * @param f
 * @param itemsFromPlain
 * @param x candidate
 * @return operation-based AVL list from `x`, or undefined if `x` is mal-formed
 */
export const opListFromPlain = OpLinkedList.fromPlain;
/**
 * @param factory block factory
 * @param v empty value (used for type inference)
 * @return operation-based empty linked list that cannot be locally modified
 */
export function OpEditableList(factory, v) {
    assert(() => v.length === 0, "v must be empty");
    return EditableOpLinkedList.emptyWith(factory);
}
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
export const opEditableListFromPlain = EditableOpLinkedList.fromPlain;
/**
 * @return operation-based empty linked list that can be locally modified
 */
export function deltaList() {
    return DeltaReplicatedList.from(opList());
}
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
export function deltaListFromPlain(f, itemsFromPlain) {
    return DeltaReplicatedList.fromPlain(opListFromPlain(f, itemsFromPlain));
}
/**
 * @param factory block factory
 * @param v empty value (used for type inference)
 * @return delta-based empty linked list that can be locally modified
 */
export function deltaEditableList(factory, v) {
    assert(() => v.length === 0, "v must be empty");
    return DeltaEditableReplicatedList.from(OpEditableList(factory, v));
}
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
export function deltaEditableListFromPlain(f, itemsFromPlain) {
    return DeltaEditableReplicatedList.fromPlain(opEditableListFromPlain(f, itemsFromPlain));
}
//# sourceMappingURL=linked.js.map