/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
/**
 * Inversion of Ordering.
 *
 * @example
 * orderingInversion[Ordering.BEFORE] === Ordering.AFTER
 */
export const orderingInversion = {
    [-1 /* BEFORE */]: 1 /* AFTER */,
    [0 /* EQUAL */]: 0 /* EQUAL */,
    [1 /* AFTER */]: -1 /* BEFORE */,
};
/**
 * compareBoolean(false, true) === Ordering.EQUAL
 * @param a
 * @param b
 * @return a [ordering relation] b
 */
export const compareBoolean = (a, b) => {
    if (a === b) {
        return 0 /* EQUAL */;
    }
    else if (b) {
        return 1 /* AFTER */;
    }
    else {
        return -1 /* BEFORE */;
    }
};
/**
 * @example
 * lexCompareOrdering(Ordering.Equal, r2) == r2
 * lexCompareOrdering(Ordering.Before, _) == Ordering.Before
 * lexCompareOrdering(Ordering.After, _) == Ordering.After
 *
 * @param r1
 * @param r2
 * @return Lexicographic order between {@link r1} and {@link r2}.
 */
export const lexCompareOrdering = (r1, r2) => {
    return r1 !== 0 /* EQUAL */ ? r1 : r2;
};
//# sourceMappingURL=ordering.js.map