import test from "ava";
import { twinBlock } from "../dot-pos/simple/pos.testutil";
import { Block } from "../../src/core/block";
import { SimpleDotPos } from "../../src/dot-pos/simple/simple-dot-pos";
import { SimpleDotPosPart } from "../../src/dot-pos/simple/simple-dot-pos-part";
const part1 = SimpleDotPosPart.from(1, 0, 1);
const part3 = SimpleDotPosPart.from(1, 0, 3);
const prefixingPart1 = SimpleDotPosPart.from(1, 1, 1);
const pos1 = SimpleDotPos.from([prefixingPart1, part1]);
const pos10 = SimpleDotPos.from([prefixingPart1, part1, part3]);
const b1abc = new Block(pos1, "abc");
const b1ab = new Block(pos1, "ab");
const b1bc = new Block(pos1.intSucc(1), "bc");
const b1a = new Block(pos1, "a");
const b1b = new Block(pos1.intSucc(1), "b");
const b1c = new Block(pos1.intSucc(2), "c");
const b10xyz = new Block(pos10, "xyz");
test("length", (t) => {
    t.is(b1abc.length, b1abc.content.length);
});
test("nthPosition", (t) => {
    for (let i = 0; i < b1abc.length; i++) {
        t.deepEqual(b1abc.nthPos(i), b1abc.lowerPos.intSucc(i));
    }
});
test("upperPosition", (t) => {
    t.deepEqual(b1abc.upperPos(), b1abc.nthPos(b1abc.length - 1));
});
test("compare", (t) => {
    t.is(b1a.compare(twinBlock(b1a)), 0 /* EQUAL */);
    t.is(b10xyz.compare(b1a), 7 /* AFTER */);
    t.is(b1a.compare(b10xyz), -7 /* BEFORE */);
    t.is(b10xyz.compare(b1b), -7 /* BEFORE */);
    t.is(b1b.compare(b10xyz), 7 /* AFTER */);
    t.is(b1a.compare(b1bc), -6 /* PREPENDABLE */);
    t.is(b1bc.compare(b1a), 6 /* APPENDABLE */);
    t.is(b1a.compare(b1c), -7 /* BEFORE */);
    t.is(b1c.compare(b1a), 7 /* AFTER */);
    t.is(b1abc.compare(b10xyz), -5 /* SPLITTED_BY */);
    t.is(b10xyz.compare(b1abc), 5 /* SPLITTING */);
    t.is(b1ab.compare(b1bc), -4 /* OVERLAPPING_BEFORE */);
    t.is(b1bc.compare(b1ab), 4 /* OVERLAPPING_AFTER */);
    t.is(b1abc.compare(b1a), 1 /* INCLUDING_LEFT */);
    t.is(b1a.compare(b1abc), -3 /* INCLUDED_LEFT_BY */);
    t.is(b1abc.compare(b1c), 3 /* INCLUDING_RIGHT */);
    t.is(b1c.compare(b1abc), -1 /* INCLUDED_RIGHT_BY */);
    t.is(b1abc.compare(b1b), 2 /* INCLUDING_MIDDLE */);
    t.is(b1b.compare(b1abc), -2 /* INCLUDED_MIDDLE_BY */);
});
test("append", (t) => {
    t.deepEqual(b1a.append(b1bc), b1abc);
});
test("spliAt", (t) => {
    t.deepEqual(b1abc.splitAt(1), [b1a, b1bc]);
});
test("splittingIndex", (t) => {
    t.is(b1abc.splittingIndex(b10xyz), 1);
});
test("splitWith", (t) => {
    t.deepEqual(b1abc.splitWith(b10xyz), [b1a, b1bc]);
});
test("has-prependable", (t) => {
    t.true(b1ab.hasPrependable(b1bc));
    t.true(b1ab.hasPrependable(b1b));
    t.true(b1abc.hasPrependable(b1b));
    t.false(b1bc.hasPrependable(b1b));
    t.false(b1b.hasPrependable(b1c));
});
test("prependable", (t) => {
    t.deepEqual(b1ab.prependable(b1b), b1a);
    t.deepEqual(b1abc.prependable(b1b), b1a);
});
test("has-appendable", (t) => {
    t.true(b1bc.hasAppendable(b1ab));
    t.true(b1bc.hasAppendable(b1b));
    t.true(b1abc.hasAppendable(b1b));
    t.false(b1a.hasAppendable(b1c));
    t.false(b1bc.hasAppendable(b1c));
});
test("appendable", (t) => {
    t.deepEqual(b1bc.appendable(b1b), b1c);
    t.deepEqual(b1abc.appendable(b1b), b1c);
});
test("intersection", (t) => {
    for (const b of [b1a, b1b, b1c]) {
        t.deepEqual(b1abc.intersection(b), b);
        t.deepEqual(b.intersection(b1abc), b);
    }
});
//# sourceMappingURL=block.test.js.map