import test from "ava";
import { Block } from "../../../src/core/block";
import { SimpleDotBlockFactory } from "../../../src/";
import { SimpleDotPos } from "../../../src";
import { U32_BOTTOM } from "../../../src/util/number";
import { SimpleDotPosPart } from "../../../src/dot-pos/simple/simple-dot-pos-part";
const seed = "dotted-logootsplit";
const factoryA = SimpleDotBlockFactory.from(1, seed);
const firstA = factoryA.from("ab");
const factoryB = SimpleDotBlockFactory.from(2, seed);
const firstB = factoryB.after(firstA, "rs");
test("after_appendable-block", (t) => {
    const appendable = factoryA.copy().after(firstA, "cd");
    t.is(appendable.compare(firstA), 6 /* APPENDABLE */);
});
test("between_splitting-block", (t) => {
    const [lBlock, rBlock] = firstA.splitAt(1);
    const splitting = factoryA.copy().between(lBlock, "xy", rBlock);
    t.is(splitting.compare(firstA), 5 /* SPLITTING */);
});
test("before_other-replica", (t) => {
    const after = factoryB.copy().before("xy", firstA);
    t.is(after.compare(firstA), -7 /* BEFORE */);
});
test("after_other-replica", (t) => {
    const after = factoryB.copy().after(firstA, "xy");
    t.is(after.compare(firstA), 7 /* AFTER */);
});
test("between_surrounded-block", (t) => {
    const surrounded = factoryB.copy().between(firstA, "xy", firstB);
    t.is(surrounded.compare(firstA), 7 /* AFTER */);
    t.is(surrounded.compare(firstB), -7 /* BEFORE */);
});
test("between_dense-set-appendable", (t) => {
    const B = 2;
    const priority = firstA.lowerPos.parts[0].priority;
    const part2 = SimpleDotPosPart.from(priority + 1, B, 1);
    const block2 = new Block(SimpleDotPos.from([part2]), "x");
    const surrounded = factoryB.copy().between(firstA, "1", block2);
    const appendable = factoryA.copy().between(firstA, "c", surrounded);
    t.is(appendable.compare(firstA), 6 /* APPENDABLE */);
    t.is(appendable.compare(surrounded), -7 /* BEFORE */);
    t.is(surrounded.compare(block2), -7 /* BEFORE */);
});
test("between_variable-sized-position", (t) => {
    const [A, B] = [1, 2];
    const partA1 = SimpleDotPosPart.from(5, A, 1);
    const blockA = new Block(SimpleDotPos.from([partA1]), "a");
    const partB1 = SimpleDotPosPart.from(6, B, 1);
    const partB2 = SimpleDotPosPart.from(U32_BOTTOM + 1, B, 1);
    const blockB = new Block(SimpleDotPos.from([partB1, partB2]), "2");
    const partA2 = SimpleDotPosPart.from(7, A, 1);
    const blockA2 = new Block(SimpleDotPos.from([partA2]), "c");
    const surroundedB = factoryB.copy().between(blockA, "1", blockB);
    t.is(blockA.compare(surroundedB), -7 /* BEFORE */);
    t.is(surroundedB.compare(blockB), -7 /* BEFORE */);
    const surroundedA = factoryA.copy().between(blockB, "b", blockA2);
    t.is(blockB.compare(surroundedA), -7 /* BEFORE */);
    t.is(surroundedA.compare(blockA2), -7 /* BEFORE */);
});
test("from-plain", (t) => {
    t.is(SimpleDotBlockFactory.fromPlain(undefined), undefined);
    t.is(SimpleDotBlockFactory.fromPlain(null), undefined);
    t.is(SimpleDotBlockFactory.fromPlain({}), undefined);
    t.is(SimpleDotBlockFactory.fromPlain([]), undefined);
    t.deepEqual(SimpleDotBlockFactory.fromPlain(factoryA), factoryA);
});
//# sourceMappingURL=simple-block-factory.test.js.map