import { Ins, Del } from "../../src";
const titled = (defaultTitle) => (title = defaultTitle, ...rest) => `${rest[rest.length - 1]}_${title}`;
// empty list
export const mEmpty = (t, emp) => {
    const seqA = emp(1 /* A */);
    t.is(seqA.concatenated(""), "");
    t.is(seqA.length, 0);
};
mEmpty.title = titled("empty");
// inserAt
export const mInsertAtRight = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "ab");
    seqA.insertAt(2, "cd");
    seqA.insertAt(4, "ef");
    t.is(seqA.concatenated(""), "abcdef");
    t.is(seqA.length, 6);
};
mInsertAtRight.title = titled("insert-at-right");
export const mInsertAtLeft = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "ef");
    seqA.insertAt(0, "cd");
    seqA.insertAt(0, "ab");
    t.is(seqA.concatenated(""), "abcdef");
    t.is(seqA.length, 6);
};
mInsertAtLeft.title = titled("insert-at-left");
export const mInsertAtBoth = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "ef");
    seqA.insertAt(0, "ab");
    seqA.insertAt(4, "ij");
    seqA.insertAt(2, "cd");
    seqA.insertAt(6, "gh");
    t.is(seqA.concatenated(""), "abcdefghij");
    t.is(seqA.length, 10);
};
mInsertAtBoth.title = titled("insert-at-both");
export const mInsertAtSplit = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "ce");
    seqA.insertAt(0, "a");
    seqA.insertAt(3, "g");
    seqA.insertAt(1, "b");
    seqA.insertAt(4, "f");
    seqA.insertAt(3, "d"); // split "ce"
    t.is(seqA.concatenated(""), "abcdefg");
    t.is(seqA.length, 7);
};
mInsertAtSplit.title = titled("insert-at-between");
export const mInsertAtMultiple = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "c");
    seqA.insertAt(0, "b");
    seqA.insertAt(2, "d");
    seqA.insertAt(0, "a");
    seqA.insertAt(4, "e");
    t.is(seqA.concatenated(""), "abcde");
    t.is(seqA.length, 5);
};
mInsertAtMultiple.title = titled("insert-at-multiple");
// insert
export const mInsertSingle = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */);
    const ins = seqB.insert(ab);
    t.is(ab.content, "ab");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.deepEqual(ins, [new Ins(0, "ab")]);
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mInsertSingle.title = titled("insert-single");
export const mInsertReplayed = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    t.throws(() => seqB.insert(ab));
};
mInsertReplayed.title = titled("insert-replayed");
export const mInsertAppend = (t, emp) => {
    const seqA = emp(1 /* A */);
    const [ab, cd] = seqA.insertAt(0, "abcd").splitAt(2);
    const seqB = emp(2 /* B */);
    const abIns = seqB.insert(ab);
    const cdIns = seqB.insert(cd);
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.deepEqual(abIns, [new Ins(0, "ab")]);
    t.deepEqual(cdIns, [new Ins(2, "cd")]);
};
mInsertAppend.title = titled("insert-append");
export const mInsertPrepend = (t, emp) => {
    const seqA = emp(1 /* A */);
    const [ab, cd] = seqA.insertAt(0, "abcd").splitAt(2);
    const seqB = emp(2 /* B */);
    const cdIns = seqB.insert(cd);
    const abIns = seqB.insert(ab);
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.deepEqual(cdIns, [new Ins(0, "cd")]);
    t.deepEqual(abIns, [new Ins(0, "ab")]);
};
mInsertPrepend.title = titled("insert-prepend");
export const mInsertAppendPrepend = (t, emp) => {
    const seqA = emp(1 /* A */);
    const [ab, cdef] = seqA.insertAt(0, "abcdef").splitAt(2);
    const [cd, ef] = cdef.splitAt(2);
    const seqB = emp(2 /* B */);
    const abIns = seqB.insert(ab);
    const efIns = seqB.insert(ef);
    const cdIns = seqB.insert(cd);
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.deepEqual(abIns, [new Ins(0, "ab")]);
    t.deepEqual(efIns, [new Ins(2, "ef")]);
    t.deepEqual(cdIns, [new Ins(2, "cd")]);
};
mInsertAppendPrepend.title = titled("insert-append-prepend");
export const mInsertSplitting = (t, emp) => {
    const seqA = emp(1 /* A */);
    const bd = seqA.insertAt(0, "bd");
    const a = seqA.insertAt(0, "a");
    const e = seqA.insertAt(3, "e");
    const c = seqA.insertAt(2, "c");
    const seqB = emp(2 /* B */);
    seqB.insert(bd);
    seqB.insert(a);
    seqB.insert(e);
    const cIns = seqB.insert(c);
    t.is(seqA.concatenated(""), "abcde");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.is(seqA.length, 5);
    t.deepEqual(cIns, [new Ins(2, "c")]);
};
mInsertSplitting.title = titled("insert-splitting");
export const mInsertSplitted = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ac = seqA.insertAt(0, "ac");
    const b = seqA.insertAt(1, "b");
    const seqB = emp(2 /* B */);
    seqB.insert(b);
    const acIns = seqB.insert(ac);
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.is(seqA.length, 3);
    let exepctedIns;
    if (acIns.length > 1 && acIns[0].content === "c") {
        exepctedIns = [new Ins(1, "c"), new Ins(0, "a")];
    }
    else {
        exepctedIns = [new Ins(0, "a"), new Ins(2, "c")];
    }
    t.deepEqual(acIns, exepctedIns);
};
mInsertSplitted.title = titled("insert-splitted");
export const mInsertDoublySplitted = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ace = seqA.insertAt(0, "ace");
    const d = seqA.insertAt(2, "d");
    const b = seqA.insertAt(1, "b");
    const seqB = emp(2 /* B */);
    seqB.insert(b);
    seqB.insert(d);
    const aceIns = seqB.insert(ace); // doubly splitted
    t.is(seqA.concatenated(""), "abcde");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    let exepctedIns;
    if (aceIns.length > 1 && aceIns[0].content === "e") {
        exepctedIns = [new Ins(2, "e"), new Ins(1, "c"), new Ins(0, "a")];
    }
    else {
        exepctedIns = [new Ins(0, "a"), new Ins(2, "c"), new Ins(4, "e")];
    }
    t.deepEqual(aceIns, exepctedIns);
};
mInsertDoublySplitted.title = titled("insert-doubly-splitted");
export const mInsertAppendSplitted = (t, emp) => {
    const seqA = emp(1 /* A */);
    const [ab, ce] = seqA.insertAt(0, "abce").splitAt(2);
    const d = seqA.insertAt(3, "d"); // split "ce"
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    seqB.insert(d);
    const ceIns = seqB.insert(ce); // splitted by "d"
    t.is(seqA.concatenated(""), "abcde");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    let exepctedIns;
    if (ceIns.length > 1 && ceIns[0].content === "e") {
        exepctedIns = [new Ins(3, "e"), new Ins(2, "c")];
    }
    else {
        exepctedIns = [new Ins(2, "c"), new Ins(4, "e")];
    }
    t.deepEqual(ceIns, exepctedIns);
};
mInsertAppendSplitted.title = titled("insert-append-splitted");
export const mInsertPrependSplitted = (t, emp) => {
    const seqA = emp(1 /* A */);
    const [ac, de] = seqA.insertAt(0, "acde").splitAt(2);
    const b = seqA.insertAt(1, "b"); // split "de"
    const seqB = emp(2 /* B */);
    seqB.insert(de);
    seqB.insert(b);
    const acIns = seqB.insert(ac); // splitted by "b"
    t.is(seqA.concatenated(""), "abcde");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    let exepctedIns;
    if (acIns.length > 1 && acIns[0].content === "c") {
        exepctedIns = [new Ins(1, "c"), new Ins(0, "a")];
    }
    else {
        exepctedIns = [new Ins(0, "a"), new Ins(2, "c")];
    }
    t.deepEqual(acIns, exepctedIns);
};
mInsertPrependSplitted.title = titled("insert-prepend-splitted");
export const mInsertIncluded = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcdef = seqA.insertAt(0, "abcdef");
    const cd = abcdef.rightSplitAt(2).leftSplitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(abcdef);
    t.throws(() => seqB.insert(cd));
};
mInsertIncluded.title = titled("insert-included");
export const mInsertIncluding = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcdef = seqA.insertAt(0, "abcdef");
    const cd = abcdef.rightSplitAt(2).leftSplitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(cd);
    t.throws(() => seqB.insert(abcdef));
};
mInsertIncluding.title = titled("insert-including");
export const mInsertOverlappingAfter = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    const ab = abc.leftSplitAt(2);
    const bc = abc.rightSplitAt(1);
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    t.throws(() => seqB.insert(bc));
};
mInsertOverlappingAfter.title = titled("insert-overlapping-after");
export const mInsertOverlappingBefore = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    const ab = abc.leftSplitAt(2);
    const bc = abc.rightSplitAt(1);
    const seqB = emp(2 /* B */);
    seqB.insert(bc);
    t.throws(() => seqB.insert(ab));
};
mInsertOverlappingBefore.title = titled("insert-overlapping-before");
// insert by separate peers
export const mInsertAfter = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    const _12 = seqB.insertAt(2, "12");
    seqA.insert(_12);
    t.is(seqA.concatenated(""), "ab12");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mInsertAfter.title = titled("insert-after");
export const mInsertBefore = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    const _12 = seqB.insertAt(0, "12");
    seqA.insert(_12);
    t.is(seqA.concatenated(""), "12ab");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mInsertBefore.title = titled("insert-before");
// concurrent insert
export const mInsertConcurrent = (t, emp) => {
    const seed = "seed";
    const seqA = emp(1 /* A */, seed);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */, seed);
    const _12 = seqB.insertAt(0, "12");
    seqA.insert(_12);
    seqB.insert(ab);
    t.is(seqA.concatenated(""), "ab12");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mInsertConcurrent.title = titled("insert-concurrent");
export const mInsertConcurrentAppend = (t, emp) => {
    const seed = "seed";
    const seqA = emp(1 /* A */, seed);
    const ab = seqA.insertAt(0, "ab");
    const cd = seqA.insertAt(2, "cd");
    const seqB = emp(2 /* B */, seed);
    const _12 = seqB.insertAt(0, "12");
    const _34 = seqB.insertAt(2, "34");
    seqB.insert(ab);
    seqB.insert(cd);
    seqA.insert(_12);
    seqA.insert(_34);
    t.is(seqA.concatenated(""), "abcd1234");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mInsertConcurrentAppend.title = titled("insert-concurrent-append");
export const mInsertConcurrentPrepend = (t, emp) => {
    const seed = "seed";
    const seqA = emp(1 /* A */, seed);
    const ab = seqA.insertAt(0, "ab");
    const cd = seqA.insertAt(2, "cd");
    const seqB = emp(2 /* B */, seed);
    const _12 = seqB.insertAt(0, "12");
    const _34 = seqB.insertAt(2, "34");
    seqB.insert(cd);
    seqB.insert(ab);
    seqA.insert(_34);
    seqA.insert(_12);
    t.is(seqA.concatenated(""), "abcd1234");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mInsertConcurrentPrepend.title = titled("insert-concurrent-prepend");
// removeAt
export const mRemoveAtEqual = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "abc");
    seqA.removeAt(0, 3); // remove "abc"
    t.is(seqA.concatenated(""), "");
    t.is(seqA.length, 0);
};
mRemoveAtEqual.title = titled("remove-at-equal");
export const mRemoveAtEqualMerge = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const seqB = emp(2 /* B */);
    seqB.insert(abcd);
    seqB.insertAt(2, "12"); // split "abcd"
    seqB.removeAt(2, 2); // remove "12"
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mRemoveAtEqualMerge.title = titled("remove-at-equal-merge");
export const mRemoveAtJustBefore = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    seqB.insertAt(0, "12");
    seqB.removeAt(0, 2); // remove "12"
    t.is(seqA.concatenated(""), "ab");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mRemoveAtJustBefore.title = titled("remove-at-just-before");
export const mRemoveAtBefore = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    seqB.insertAt(0, "12");
    seqB.removeAt(0, 1); // remove "1"
    t.is(seqB.concatenated(""), "2ab");
};
mRemoveAtBefore.title = titled("remove-at-before");
export const mRemoveAtJustAfter = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    seqB.insertAt(2, "12");
    seqB.removeAt(2, 2); // remove "12"
    t.is(seqA.concatenated(""), "ab");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mRemoveAtJustAfter.title = titled("remove-at-just-after");
export const mRemoveAtAfter = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    seqB.insertAt(2, "12");
    seqB.removeAt(3, 1); // remove "2"
    t.is(seqB.concatenated(""), "ab1");
};
mRemoveAtAfter.title = titled("remove-at-after");
export const mRemoveAtIncludedLeftBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "abc");
    seqA.removeAt(0, 2); // remove "ab"
    t.is(seqA.concatenated(""), "c");
    t.is(seqA.length, 1);
};
mRemoveAtIncludedLeftBy.title = titled("remove-at-included-left-by");
export const mRemoveAtIncludedMiddleBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "abc");
    seqA.removeAt(1, 1); // remove "b"
    t.is(seqA.concatenated(""), "ac");
    t.is(seqA.length, 2);
};
mRemoveAtIncludedMiddleBy.title = titled("remove-at-included-middle-by");
export const mRemoveAtIncludedRightBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "abc");
    seqA.removeAt(1, 2); // remove "bc"
    t.is(seqA.concatenated(""), "a");
    t.is(seqA.length, 1);
};
mRemoveAtIncludedRightBy.title = titled("remove-at-included-right-by");
export const mRemoveAtIncludingLeft = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    const seqB = emp(2 /* B */);
    seqB.insert(abc);
    seqB.insertAt(3, "def");
    seqB.removeAt(0, 4); // remove "abc" and "d"
    t.is(seqB.concatenated(""), "ef");
    t.is(seqB.length, 2);
};
mRemoveAtIncludingLeft.title = titled("remove-at-including-left");
export const mRemoveAtIncludingMiddle = (t, emp) => {
    const seqA = emp(1 /* A */);
    const def = seqA.insertAt(0, "def");
    const seqB = emp(2 /* B */);
    seqB.insert(def);
    seqB.insertAt(3, "ghi");
    seqB.insertAt(0, "abc");
    seqB.removeAt(2, 5); // remove "c", "def", and "g"
    t.is(seqB.concatenated(""), "abhi");
};
mRemoveAtIncludingMiddle.title = titled("remove-at-including-middle");
export const mRemoveAtIncludingRight = (t, emp) => {
    const seqA = emp(1 /* A */);
    const def = seqA.insertAt(0, "def");
    const seqB = emp(2 /* B */);
    seqB.insert(def);
    seqB.insertAt(0, "abc");
    seqB.removeAt(2, 4); // remove "c" and "def"
    t.is(seqB.concatenated(""), "ab");
};
mRemoveAtIncludingRight.title = titled("remove-at-including-right");
export const mRemoveAtMultiple = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "ef");
    seqA.insertAt(0, "ab");
    seqA.insertAt(4, "ij");
    seqA.insertAt(2, "cd");
    seqA.insertAt(6, "gh");
    seqA.removeAt(1, 8);
    t.is(seqA.concatenated(""), "aj");
    t.is(seqA.length, 2);
};
mRemoveAtMultiple.title = titled("remove-at-multiple");
// remove
export const mRemoveNothing = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "ab");
    const [abRmv] = seqA.removeAt(0, 2); // remove "ab"
    const seqB = emp(2 /* B */);
    const abLocalRemoval = seqB.remove(abRmv);
    t.is(seqA.concatenated(""), "");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.deepEqual(abLocalRemoval, []);
};
mRemoveNothing.title = titled("remove-nothing");
export const mRemoveEqual = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const cdRmv = seqA.remove(abcd.toLengthBlock());
    t.is(seqA.concatenated(""), "");
    t.deepEqual(cdRmv, [new Del(0, 4)]);
};
mRemoveEqual.title = titled("remove-equal");
export const mRemoveIncludedLeftBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const ab = abcd.leftSplitAt(2);
    const abRmv = seqA.remove(ab.toLengthBlock());
    t.is(seqA.concatenated(""), "cd");
    t.deepEqual(abRmv, [new Del(0, 2)]);
};
mRemoveIncludedLeftBy.title = titled("remove-included-left-by");
export const mRemoveIncludedMiddleBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const bcd = abcd.rightSplitAt(1);
    const bc = bcd.leftSplitAt(2);
    const bcRmv = seqA.remove(bc.toLengthBlock());
    t.is(seqA.concatenated(""), "ad");
    t.deepEqual(bcRmv, [new Del(1, 2)]);
};
mRemoveIncludedMiddleBy.title = titled("remove-included-middle-by");
export const mRemoveIncludedRightBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const cd = abcd.rightSplitAt(2);
    const cdRmv = seqA.remove(cd.toLengthBlock());
    t.is(seqA.concatenated(""), "ab");
    t.deepEqual(cdRmv, [new Del(2, 2)]);
};
mRemoveIncludedRightBy.title = titled("remove-included-right-by");
export const mRemoveIncludingLeft = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const ab = abcd.leftSplitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    const abRmv = seqB.remove(abcd.toLengthBlock());
    t.is(seqB.concatenated(""), "");
    t.deepEqual(abRmv, [new Del(0, 2)]);
};
mRemoveIncludingLeft.title = titled("remove-including-left");
export const mRemoveIncludingMiddle = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const bcd = abcd.rightSplitAt(1);
    const bc = bcd.leftSplitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(bc);
    const bcRmv = seqB.remove(abcd.toLengthBlock());
    t.is(seqB.concatenated(""), "");
    t.deepEqual(bcRmv, [new Del(0, 2)]);
};
mRemoveIncludingMiddle.title = titled("remove-including-middle");
export const mRemoveIncludingRight = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const cd = abcd.rightSplitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(cd);
    const cdRmv = seqB.remove(abcd.toLengthBlock());
    t.is(seqB.concatenated(""), "");
    t.deepEqual(cdRmv, [new Del(0, 2)]);
};
mRemoveIncludingRight.title = titled("remove-including-right");
export const mRemoveOverlappingLeft = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const ab = abcd.leftSplitAt(2);
    const bcd = abcd.rightSplitAt(1);
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    const bRmv = seqB.remove(bcd.toLengthBlock());
    t.is(seqB.concatenated(""), "a");
    t.deepEqual(bRmv, [new Del(1, 1)]);
};
mRemoveOverlappingLeft.title = titled("remove-overlapping-left");
export const mRemoveOverlappingRight = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const cd = abcd.rightSplitAt(2);
    const abc = abcd.leftSplitAt(3);
    const seqB = emp(2 /* B */);
    seqB.insert(cd);
    const cRmv = seqB.remove(abc.toLengthBlock());
    t.is(seqB.concatenated(""), "d");
    t.deepEqual(cRmv, [new Del(0, 1)]);
};
mRemoveOverlappingRight.title = titled("remove-overlapping-right");
export const mRemoveSplitting = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abef = seqA.insertAt(0, "abef");
    const seqB = emp(2 /* B */);
    seqB.insert(abef);
    const cd = seqB.insertAt(2, "cd"); // split
    seqA.remove(cd.toLengthBlock()); // no effect
    seqB.remove(cd.toLengthBlock());
    t.is(seqA.concatenated(""), "abef");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mRemoveSplitting.title = titled("remove-splitting");
export const mRemoveSplittedBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abef = seqA.insertAt(0, "abef");
    const seqB = emp(2 /* B */);
    seqB.insert(abef);
    const cd = seqB.insertAt(2, "cd"); // split
    seqA.remove(abef.toLengthBlock());
    seqB.remove(abef.toLengthBlock());
    seqA.insert(cd);
    t.is(seqA.concatenated(""), "cd");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mRemoveSplittedBy.title = titled("remove-splitted-by");
export const mRemoveAfterBeforeMerge = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abefij = seqA.insertAt(0, "abefij");
    const [ab, efij] = abefij.splitAt(2);
    const [ef, ij] = efij.splitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(ef);
    const gh = seqB.insertAt(2, "gh");
    const cd = seqB.insertAt(0, "cd");
    seqB.insert(ij);
    seqB.insert(ab);
    seqB.remove(gh.toLengthBlock());
    seqB.remove(cd.toLengthBlock());
    t.is(seqA.concatenated(""), "abefij");
    t.is(seqA.concatenated(""), seqB.concatenated(""));
    t.is(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mRemoveAfterBeforeMerge.title = titled("remove-efter-before-merge");
// insertable
export const mInsertertableEqual = (t, emp) => {
    const seqA = emp(1 /* A */);
    const ab = seqA.insertAt(0, "ab");
    t.deepEqual(seqA.insertable(ab), []);
};
mInsertertableEqual.title = titled("insertable-equal");
export const mInsertertableIncludedLeftBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const ab = abcd.leftSplitAt(2);
    t.deepEqual(seqA.insertable(ab), []);
};
mInsertertableIncludedLeftBy.title = titled("insertable-included-left-by");
export const mInsertertableIncludedMiddleBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const cd = abcd.rightSplitAt(2);
    const c = cd.leftSplitAt(1);
    t.deepEqual(seqA.insertable(c), []);
};
mInsertertableIncludedMiddleBy.title = titled("insertable-included-middle-by");
export const mInsertertableIncludedRightBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const cd = abcd.rightSplitAt(2);
    t.deepEqual(seqA.insertable(cd), []);
};
mInsertertableIncludedRightBy.title = titled("insertable-included-right-by");
export const mInsertertableIncludingLeft = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const [ab, cd] = abcd.splitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(cd);
    t.deepEqual(seqB.insertable(abcd), [ab]);
};
mInsertertableIncludingLeft.title = titled("insertable-including-left");
export const mInsertertableIncludingMiddle = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const [abc, d] = abcd.splitAt(3);
    const [a, bc] = abc.splitAt(1);
    const seqB = emp(2 /* B */);
    seqB.insert(bc);
    t.deepEqual(seqB.insertable(abcd), [a, d]);
};
mInsertertableIncludingMiddle.title = titled("insertable-including-middle");
export const mInsertertableIncludingRight = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const [ab, cd] = abcd.splitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(ab);
    t.deepEqual(seqB.insertable(abcd), [cd]);
};
mInsertertableIncludingRight.title = titled("insertable-including-right");
export const mInsertertableOverlappingLeft = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const abc = abcd.leftSplitAt(3);
    const [ab, cd] = abcd.splitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(cd);
    t.deepEqual(seqB.insertable(abc), [ab]);
};
mInsertertableOverlappingLeft.title = titled("insertable-overlapping-left");
export const mInsertertableOverlappingRight = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abcd = seqA.insertAt(0, "abcd");
    const bcd = abcd.rightSplitAt(1);
    const cd = abcd.rightSplitAt(2);
    const seqB = emp(2 /* B */);
    seqB.insert(cd);
    t.deepEqual(seqB.insertable(bcd), [cd]);
};
mInsertertableOverlappingRight.title = titled("insertable-overlapping-right");
export const mInsertertableSplittedBy = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abef = seqA.insertAt(0, "abef");
    const [ab, ef] = abef.splitAt(2);
    const cd = seqA.insertAt(2, "cd"); // split
    const seqB = emp(2 /* B */);
    seqB.insert(cd);
    t.deepEqual(seqB.insertable(abef), [ab, ef]);
};
mInsertertableSplittedBy.title = titled("insertable-splitted-by");
export const mInsertertableSplitting = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abef = seqA.insertAt(0, "abef");
    const cd = seqA.insertAt(2, "cd"); // split
    const seqB = emp(2 /* B */);
    seqB.insert(abef);
    t.deepEqual(seqB.insertable(cd), [cd]);
};
mInsertertableSplitting.title = titled("insertable-splitting");
export const mInsertertableAfterBefore = (t, emp) => {
    const seqA = emp(1 /* A */);
    const cd = seqA.insertAt(0, "cd");
    const seqB = emp(2 /* B */);
    seqB.insert(cd);
    const ef = seqB.insertAt(2, "ef");
    const ab = seqB.insertAt(0, "ab");
    t.deepEqual(seqA.insertable(ab), [ab]);
    t.deepEqual(seqA.insertable(ef), [ef]);
};
mInsertertableAfterBefore.title = titled("insertable-efter-before");
export const mInsertertableAppednable = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    const [a, bc] = abc.splitAt(1);
    const [b, c] = bc.splitAt(1);
    const seqB = emp(2 /* B */);
    seqB.insert(b);
    t.deepEqual(seqB.insertable(a), [a]);
    t.deepEqual(seqB.insertable(c), [c]);
};
mInsertertableAppednable.title = titled("insertable-appendable");
// applyDelta
export const mApplyDeltaTwice = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    seqA.applyDelta(abc);
    const seqB = emp(2 /* B */);
    seqB.applyDelta(abc);
    seqB.applyDelta(abc);
    t.deepEqual(seqA.concatenated(""), "abc");
    t.deepEqual(seqA.concatenated(""), seqB.concatenated(""));
    t.deepEqual(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mApplyDeltaTwice.title = titled("apply-delta-twice");
export const mApplyDeltaRemoveInsert = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    seqA.applyDelta(abc.toLengthBlock());
    const seqB = emp(2 /* B */);
    seqB.applyDelta(abc.toLengthBlock());
    seqB.applyDelta(abc);
    t.deepEqual(seqA.concatenated(""), "");
    t.deepEqual(seqA.concatenated(""), seqB.concatenated(""));
    t.deepEqual(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mApplyDeltaRemoveInsert.title = titled("apply-delta-remove-insert");
export const mApplyDeltaInsertRemove = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    seqA.applyDelta(abc.toLengthBlock());
    const seqB = emp(2 /* B */);
    seqB.applyDelta(abc);
    seqB.applyDelta(abc.toLengthBlock());
    t.deepEqual(seqA.concatenated(""), "");
    t.deepEqual(seqA.concatenated(""), seqB.concatenated(""));
    t.deepEqual(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mApplyDeltaInsertRemove.title = titled("apply-delta-insert-remove");
export const mApplyDeltaPartRemoveInsert = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    const a = abc.leftSplitAt(1);
    seqA.applyDelta(a.toLengthBlock());
    const seqB = emp(2 /* B */);
    seqB.applyDelta(a.toLengthBlock());
    seqB.applyDelta(abc);
    t.deepEqual(seqA.concatenated(""), "bc");
    t.deepEqual(seqA.concatenated(""), seqB.concatenated(""));
    t.deepEqual(seqA.structuralHashCode(), seqB.structuralHashCode());
};
mApplyDeltaPartRemoveInsert.title = titled("apply-delta-part-remove-insert");
// merge
export const mMergeSimple = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "abc");
    const seqB = emp(2 /* B */);
    const opB = seqB.merge(seqA);
    t.deepEqual(seqA.concatenated(""), "abc");
    t.deepEqual(seqA.concatenated(""), seqB.concatenated(""));
    t.deepEqual(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.deepEqual(opB, [new Ins(0, "abc")]);
};
mMergeSimple.title = titled("merge-simple");
export const mMergeIdempotent = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "abc");
    const opA = seqA.merge(seqA);
    const seqB = emp(2 /* B */);
    const opB = seqB.merge(seqA);
    t.deepEqual(seqA.concatenated(""), "abc");
    t.deepEqual(seqA.concatenated(""), seqB.concatenated(""));
    t.deepEqual(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.deepEqual(opA, []);
    t.deepEqual(opB, [new Ins(0, "abc")]);
};
mMergeIdempotent.title = titled("merge-simple-idempotent");
export const mMerge = (t, emp) => {
    const seqA = emp(1 /* A */);
    const abc = seqA.insertAt(0, "abc");
    const seqB = emp(2 /* B */);
    seqB.applyDelta(abc);
    const def = seqB.insertAt(3, "def");
    seqB.insertAt(6, "g");
    seqB.removeAt(0, 1);
    seqA.applyDelta(def);
    seqA.removeAt(5, 1);
    const opA = seqA.merge(seqB);
    const opB = seqB.merge(seqA);
    t.deepEqual(seqA.concatenated(""), "bcdeg");
    t.deepEqual(seqA.concatenated(""), seqB.concatenated(""));
    t.deepEqual(seqA.structuralHashCode(), seqB.structuralHashCode());
    t.deepEqual(opA, [new Ins(5, "g"), new Del(0, 1)]);
    t.deepEqual(opB, [new Del(4, 1)]);
};
mMerge.title = titled("merge");
// anchorAt / indexFrom
export const mTopBottomAnchor = (t, emp) => {
    const seqA = emp(1 /* A */);
    const bottomAnchor = seqA.anchorAt(0, false);
    const topAnchor = seqA.anchorAt(0, true);
    t.is(bottomAnchor.compare(topAnchor), -1 /* BEFORE */);
    t.is(seqA.indexFrom(bottomAnchor), 0);
    t.is(seqA.indexFrom(topAnchor), 0);
    seqA.insertAt(0, "ab");
    t.deepEqual(seqA.anchorAt(0, false), bottomAnchor);
    t.deepEqual(seqA.anchorAt(2, true), topAnchor);
    t.is(seqA.indexFrom(bottomAnchor), 0);
    t.is(seqA.indexFrom(topAnchor), 2);
};
mTopBottomAnchor.title = titled("anchor_top-bottom");
export const mAnchorOutOfBound = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "ab");
    const bAnchorAfter = seqA.anchorAt(5, false);
    const topAnchor = seqA.anchorAt(5, true);
    t.is(bAnchorAfter.compare(topAnchor), -1 /* BEFORE */);
    t.is(seqA.indexFrom(bAnchorAfter), 2);
    t.is(seqA.indexFrom(topAnchor), 2);
};
mAnchorOutOfBound.title = titled("anchor_out-of-bound");
export const mAnchor = (t, emp) => {
    const seqA = emp(1 /* A */);
    seqA.insertAt(0, "ce");
    const cAnchorBefore = seqA.anchorAt(0, true);
    const eAnchorAfter = seqA.anchorAt(2, false);
    t.is(cAnchorBefore.compare(eAnchorAfter), -1 /* BEFORE */);
    t.is(seqA.indexFrom(cAnchorBefore), 0);
    t.is(seqA.indexFrom(eAnchorAfter), 2);
    seqA.insertAt(2, "fg");
    seqA.insertAt(1, "d"); // split
    seqA.insertAt(0, "ab");
    t.is(seqA.indexFrom(cAnchorBefore), 2);
    t.is(seqA.indexFrom(eAnchorAfter), 5);
};
mAnchor.title = titled("anchor");
//# sourceMappingURL=macro.js.map