"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("../util/assert");
var number_1 = require("../util/number");
var data_validation_1 = require("../util/data-validation");
/**
 * Minimal implementation of Concatenable.
 * This enables to model a number of elements without their availability.
 *
 * For instance ConcatLength(3) can represent "abc", [1, 2, 3], or any
 * 3-elements list.
 */
var ConcatLength = /** @class */ (function () {
    /**
     * @param length {@link Concat#length}
     */
    function ConcatLength(length) {
        this.length = length;
        assert_1.assert(function () { return number_1.isU32(length); }, "length ∈ u32");
    }
    /**
     * @param x candidate
     * @return object from `x', or undefined if `x' is not valid.
     */
    ConcatLength.fromPlain = function (x) {
        if (data_validation_1.isObject(x) && number_1.isU32(x.length)) {
            return new ConcatLength(x.length);
        }
        return undefined;
    };
    // Access
    /** @override */
    ConcatLength.prototype.slice = function (lower, excludedUpper) {
        var _this = this;
        assert_1.assert(function () { return number_1.isU32(lower); }, "lower ∈ u32");
        assert_1.assert(function () { return lower < _this.length; }, "lower < this.length");
        assert_1.assert(function () { return number_1.isU32(excludedUpper); }, "excludedUpper ∈ u32");
        assert_1.assert(function () { return excludedUpper <= _this.length; }, "excludedUpper <= this.length");
        assert_1.assert(function () { return lower <= excludedUpper; }, "lower <= excludedUpper");
        return new ConcatLength(excludedUpper - lower);
    };
    /** @override */
    ConcatLength.prototype.concat = function (other) {
        var _this = this;
        assert_1.assert(function () { return number_1.isU32(_this.length + other.length); }, "valid concatenation");
        return new ConcatLength(this.length + other.length);
    };
    return ConcatLength;
}());
exports.ConcatLength = ConcatLength;
//# sourceMappingURL=concat.js.map