"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("../util/assert");
var number_1 = require("../util/number");
/**
 * Model an insertion in a single-user list.
 */
var Ins = /** @class */ (function () {
    /**
     * @param index Insertion index.
     * @param content Inserted elements.
     */
    function Ins(index, content) {
        this.index = index;
        this.content = content;
        assert_1.assert(function () { return number_1.isU32(index); }, "index ∈ u32");
        assert_1.assert(function () { return content.length > 0; }, "length ∈ u32");
        this.index = index;
        this.content = content;
    }
    Object.defineProperty(Ins.prototype, "length", {
        // Access
        /**
         * Number of inserted elements.
         */
        get: function () {
            return this.content.length;
        },
        enumerable: true,
        configurable: true
    });
    Ins.prototype.endIndex = function () {
        return this.index + this.length;
    };
    return Ins;
}());
exports.Ins = Ins;
/**
 * Model a removal in a single-user list.
 */
var Del = /** @class */ (function () {
    /**
     * @param index Removal index.
     * @param items Number of removed elements.
     */
    function Del(index, length) {
        this.index = index;
        this.length = length;
        assert_1.assert(function () { return number_1.isU32(index); }, "index ∈ u32");
        assert_1.assert(function () { return number_1.isU32(length); }, "length ∈ u32");
        assert_1.assert(function () { return length > 0; }, "length > 0");
    }
    Del.prototype.endIndex = function () {
        return this.index + this.length;
    };
    return Del;
}());
exports.Del = Del;
//# sourceMappingURL=local-operation.js.map