"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var replayable_random_1 = require("replayable-random");
var assert_1 = require("../../util/assert");
var data_validation_1 = require("../../util/data-validation");
var simple_dot_pos_1 = require("./simple-dot-pos");
var simple_dot_pos_part_1 = require("./simple-dot-pos-part");
var number_1 = require("../../util/number");
var block_factory_1 = require("../../core/block-factory");
var block_1 = require("../../core/block");
/**
 * @param values
 * @param defaultValue value to emit once {@link values } were emitted.
 * @return Infinite stream of values.
 */
function infiniteSequence(values, defaultValue) {
    var values_1, values_1_1, v, e_1_1;
    var e_1, _a;
    return tslib_1.__generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _b.trys.push([0, 5, 6, 7]);
                values_1 = tslib_1.__values(values), values_1_1 = values_1.next();
                _b.label = 1;
            case 1:
                if (!!values_1_1.done) return [3 /*break*/, 4];
                v = values_1_1.value;
                return [4 /*yield*/, v];
            case 2:
                _b.sent();
                _b.label = 3;
            case 3:
                values_1_1 = values_1.next();
                return [3 /*break*/, 1];
            case 4: return [3 /*break*/, 7];
            case 5:
                e_1_1 = _b.sent();
                e_1 = { error: e_1_1 };
                return [3 /*break*/, 7];
            case 6:
                try {
                    if (values_1_1 && !values_1_1.done && (_a = values_1.return)) _a.call(values_1);
                }
                finally { if (e_1) throw e_1.error; }
                return [7 /*endfinally*/];
            case 7:
                if (!true) return [3 /*break*/, 9];
                return [4 /*yield*/, defaultValue];
            case 8:
                _b.sent();
                return [3 /*break*/, 7];
            case 9: return [2 /*return*/];
        }
    });
}
var DEFAULT_SEQ = 1;
/**
 * Factory of block with {@link SimplePos } as implementation of
 * {@link Pos}.
 * The strategy of generation is to genearte random positions between two
 * defined positions.
 */
var SimpleDotBlockFactory = /** @class */ (function (_super) {
    tslib_1.__extends(SimpleDotBlockFactory, _super);
    /**
     * @param posBounds {@link SimpleBlockFactory#replica }
     * @param seq {@link SimpleBlockFactory#seq }
     * @param randState random generator's state
     */
    function SimpleDotBlockFactory(replica, seq, randState) {
        var _this = this;
        assert_1.assert(function () { return number_1.isU32(replica); }, "replica ∈ u32");
        assert_1.assert(function () { return number_1.isU32(seq); }, "seq ∈ u32");
        assert_1.assert(function () { return seq !== 0; }, "seq != 0. Reserved for convenience");
        assert_1.assert(function () { return replica !== number_1.U32_TOP; }, "replica != U32_TOP. This is reserved for BOTTOM and TOP pos.");
        _this = _super.call(this, simple_dot_pos_1.SimpleDotPos) || this;
        _this.replica = replica;
        _this.seq = seq;
        _this.randState = randState;
        return _this;
    }
    /**
     * The random generator is seeded with both globalSeed and replica.
     *
     * @param replica {@link SimpleBlockFactory#replica }
     * @param globalSeed Seed for predictable random generation.
     * @return New factory with 0 as {@link SimpleBlockFactory#seq }
     */
    SimpleDotBlockFactory.from = function (replica, globalSeed) {
        assert_1.assert(function () { return number_1.isU32(replica); }, "replica ∈ u32");
        assert_1.assert(function () { return replica !== number_1.U32_TOP; }, "replica != U32_TOP. This is reserved for BOTTOM and TOP positions.");
        var seed = "" + globalSeed + replica;
        var randState = replayable_random_1.alea.mutFrom(seed);
        return new SimpleDotBlockFactory(replica, DEFAULT_SEQ, randState);
    };
    /**
     * @param x candidate
     * @return object from `x', or undefined if `x' is not valid.
     */
    SimpleDotBlockFactory.fromPlain = function (x) {
        if (data_validation_1.isObject(x) &&
            number_1.isU32(x.replica) &&
            x.replica !== number_1.U32_TOP &&
            number_1.isU32(x.seq) &&
            x.seq !== 0) {
            var mutRand = replayable_random_1.alea.mutFromPlain(x.randState);
            if (mutRand !== undefined) {
                return new SimpleDotBlockFactory(x.replica, x.seq, mutRand);
            }
        }
        return undefined;
    };
    /**
     * See {@link BlockFactoryConstructor#blockFromPlain}
     */
    SimpleDotBlockFactory.blockFromPlain = function (g) {
        return block_1.Block.fromPlain(simple_dot_pos_1.SimpleDotPos.fromPlain, g);
    };
    /**
     * @return Deep copy of this.
     */
    SimpleDotBlockFactory.prototype.copy = function () {
        var copiedRand = replayable_random_1.alea.mutFromPlain(this.randState); // FIXME
        return new SimpleDotBlockFactory(this.replica, this.seq, copiedRand);
    };
    /**
     * Increase {@link SimpleBlockFactory#seq } by {@link by }
     *
     * @param by
     */
    SimpleDotBlockFactory.prototype.increaseSeq = function (by) {
        var _this = this;
        assert_1.assert(function () { return number_1.isU32(by); }, "by ∈ u32");
        assert_1.assert(function () { return number_1.isU32(_this.seq + by); }, "no overflow");
        this.seq = this.seq + by;
    };
    /**
     * @param l
     * @param Can `length` be appended to `l`?
     */
    SimpleDotBlockFactory.prototype.isAppendable = function (l, length) {
        return l.replica() === this.replica && l.seq() + 1 === this.seq;
    };
    // Impl
    /** @override */
    SimpleDotBlockFactory.prototype.posBetween = function (l, length, u) {
        var _this = this;
        assert_1.heavyAssert(function () { return l.compare(u) === -1 /* BEFORE */; }, "l < u");
        assert_1.assert(function () { return number_1.isU32(length); }, "length ∈ u32");
        assert_1.assert(function () { return length > 0; }, "length is strictly positive");
        assert_1.assert(function () { return number_1.isU32(_this.seq + length); }, "no overflow");
        if (this.isAppendable(l, length)) {
            this.seq = this.seq + length;
            return l.intSucc(1);
        }
        else {
            var seqL = infiniteSequence(l.parts, simple_dot_pos_part_1.SimpleDotPosPart.BOTTOM);
            var seqU = infiniteSequence(u.parts, simple_dot_pos_part_1.SimpleDotPosPart.TOP);
            var parts = [];
            var partL = seqL.next().value;
            var partU = seqU.next().value;
            while (partU.priority - partL.priority < 2) {
                // Cannot insert a new part between partL and partU
                if (partL.replica === partU.replica) {
                    // Split
                    parts.push(partL);
                }
                else {
                    parts.push(partL.withSeq(number_1.U32_TOP)); // Let room for append
                }
                partL = seqL.next().value;
                partU = seqU.next().value;
            }
            var priority = replayable_random_1.distrib.mutU32Between(partL.priority + 1)(partU.priority)(this.randState);
            // priority ∈ ]partL.priority, partU.priority[
            // partL.priority exclusion ensures a dense set
            parts.push(simple_dot_pos_part_1.SimpleDotPosPart.from(priority, this.replica, this.seq));
            this.seq = this.seq + length;
            return simple_dot_pos_1.SimpleDotPos.from(parts);
        }
    };
    return SimpleDotBlockFactory;
}(block_factory_1.BlockFactory));
exports.SimpleDotBlockFactory = SimpleDotBlockFactory;
//# sourceMappingURL=simple-dot-block-factory.js.map