"use strict";
/*
    Copyright (C) 2019  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var op_linked_list_1 = require("./op-linked-list");
var assert_1 = require("../../util/assert");
var delta_replicated_list_1 = require("../../core/delta-replicated-list");
function opList() {
    return op_linked_list_1.OpLinkedList.empty();
}
exports.opList = opList;
/**
 * @param f
 * @param itemsFromPlain
 * @param x candidate
 * @return operation-based AVL list from `x`, or undefined if `x` is mal-formed
 */
exports.opListFromPlain = op_linked_list_1.OpLinkedList.fromPlain;
/**
 * @param factory block factory
 * @param v empty value (used for type inference)
 * @return operation-based empty linked list that cannot be locally modified
 */
function OpEditableList(factory, v) {
    assert_1.assert(function () { return v.length === 0; }, "v must be empty");
    return op_linked_list_1.EditableOpLinkedList.emptyWith(factory);
}
exports.OpEditableList = OpEditableList;
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
exports.opEditableListFromPlain = op_linked_list_1.EditableOpLinkedList.fromPlain;
/**
 * @return operation-based empty linked list that can be locally modified
 */
function deltaList() {
    return delta_replicated_list_1.DeltaReplicatedList.from(opList());
}
exports.deltaList = deltaList;
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
function deltaListFromPlain(f, itemsFromPlain) {
    return delta_replicated_list_1.DeltaReplicatedList.fromPlain(exports.opListFromPlain(f, itemsFromPlain));
}
exports.deltaListFromPlain = deltaListFromPlain;
/**
 * @param factory block factory
 * @param v empty value (used for type inference)
 * @return delta-based empty linked list that can be locally modified
 */
function deltaEditableList(factory, v) {
    assert_1.assert(function () { return v.length === 0; }, "v must be empty");
    return delta_replicated_list_1.DeltaEditableReplicatedList.from(OpEditableList(factory, v));
}
exports.deltaEditableList = deltaEditableList;
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
function deltaEditableListFromPlain(f, itemsFromPlain) {
    return delta_replicated_list_1.DeltaEditableReplicatedList.fromPlain(exports.opEditableListFromPlain(f, itemsFromPlain));
}
exports.deltaEditableListFromPlain = deltaEditableListFromPlain;
//# sourceMappingURL=linked.js.map