"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var replayable_random_1 = require("replayable-random");
var assert_1 = require("../../util/assert");
var data_validation_1 = require("../../util/data-validation");
var simple_pos_1 = require("./simple-pos");
var simple_pos_part_1 = require("./simple-pos-part");
var number_1 = require("../../util/number");
var block_factory_1 = require("../../core/block-factory");
var block_1 = require("../../core/block");
var u32_range_1 = require("../../core/u32-range");
var U32_MID = 2147483647;
/**
 * @param values
 * @param defaultValue value to emit once {@link values } were emitted.
 * @return Infinite stream of values.
 */
function infiniteSequence(values, defaultValue) {
    var values_1, values_1_1, v, e_1_1;
    var e_1, _a;
    return tslib_1.__generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _b.trys.push([0, 5, 6, 7]);
                values_1 = tslib_1.__values(values), values_1_1 = values_1.next();
                _b.label = 1;
            case 1:
                if (!!values_1_1.done) return [3 /*break*/, 4];
                v = values_1_1.value;
                return [4 /*yield*/, v];
            case 2:
                _b.sent();
                _b.label = 3;
            case 3:
                values_1_1 = values_1.next();
                return [3 /*break*/, 1];
            case 4: return [3 /*break*/, 7];
            case 5:
                e_1_1 = _b.sent();
                e_1 = { error: e_1_1 };
                return [3 /*break*/, 7];
            case 6:
                try {
                    if (values_1_1 && !values_1_1.done && (_a = values_1.return)) _a.call(values_1);
                }
                finally { if (e_1) throw e_1.error; }
                return [7 /*endfinally*/];
            case 7:
                if (!true) return [3 /*break*/, 9];
                return [4 /*yield*/, defaultValue];
            case 8:
                _b.sent();
                return [3 /*break*/, 7];
            case 9: return [2 /*return*/];
        }
    });
}
/**
 * Factory of block with {@link SimplePos } as implementation of
 * {@link Pos}.
 * The strategy of generation is to genearte random positions between two
 * defined positions.
 */
var SimpleBlockFactory = /** @class */ (function (_super) {
    tslib_1.__extends(SimpleBlockFactory, _super);
    /**
     * @param posBounds {@link SimpleBlockFactory#replica }
     * @param seq {@link SimpleBlockFactory#seq }
     * @param randState random generator's state
     */
    function SimpleBlockFactory(replica, seq, randState, generated) {
        var _this = this;
        assert_1.assert(function () { return number_1.isU32(replica); }, "replica ∈ u32");
        assert_1.assert(function () { return number_1.isU32(seq); }, "seq ∈ u32");
        assert_1.assert(function () { return replica !== number_1.U32_TOP; }, "replica != U32_TOP. This is reserved for BOTTOM and TOP pos.");
        _this = _super.call(this, simple_pos_1.SimplePos) || this;
        _this.replica = replica;
        _this.seq = seq;
        _this.randState = randState;
        _this.generated = generated;
        return _this;
    }
    /**
     * The random generator is seeded with both globalSee>eplica.
     *
     * @param replica {@link SimpleBlockFactory#replica }>
     * @param globalSeed Seed for predictable random generation.
     * @return New factory with 0 as {@link SimpleBlockFactory#seq }
     */
    SimpleBlockFactory.from = function (replica, globalSeed) {
        assert_1.assert(function () { return number_1.isU32(replica); }, "replica ∈ u32");
        assert_1.assert(function () { return replica !== number_1.U32_TOP; }, "replica != U32_TOP. This is reserved for BOTTOM and TOP positions.");
        var seed = "" + globalSeed + replica;
        var randState = replayable_random_1.alea.mutFrom(seed);
        return new SimpleBlockFactory(replica, 0, randState, new Map());
    };
    /**
     * @param x candidate
     * @return object from `x', or undefined if `x' is not valid.
     */
    SimpleBlockFactory.fromPlain = function (x) {
        if (data_validation_1.isObject(x) &&
            number_1.isU32(x.replica) &&
            x.replica !== number_1.U32_TOP &&
            number_1.isU32(x.seq) &&
            x.generated instanceof Array) {
            var generated = x.generated
                .map(function (y) {
                if (y instanceof Array &&
                    y.length === 2 &&
                    number_1.isU32(y[0]) &&
                    y[1] instanceof Array &&
                    y[1].length === 2 &&
                    number_1.isU32(y[1][1])) {
                    var _a = tslib_1.__read(y, 2), key = _a[0], _b = tslib_1.__read(_a[1], 2), maybeRange = _b[0], removalCOunt = _b[1];
                    var range = u32_range_1.U32Range.fromPlain(maybeRange);
                    if (range !== undefined) {
                        return [key, [range, removalCOunt]];
                    }
                }
                return undefined;
            })
                .filter(function (e) { return e !== undefined; });
            var mutG = replayable_random_1.alea.mutFromPlain(x.randState);
            if (x.generated.length === generated.length && mutG !== undefined) {
                // FIXME: check randState
                return new SimpleBlockFactory(x.replica, x.seq, mutG, new Map(generated));
            }
        }
        return undefined;
    };
    /**
     * See {@link BlockFactoryConstructor#blockFromPlain}
     */
    SimpleBlockFactory.blockFromPlain = function (g) {
        return block_1.Block.fromPlain(simple_pos_1.SimplePos.fromPlain, g);
    };
    // Access
    /**
     * @return Deep copy of this.
     */
    SimpleBlockFactory.prototype.copy = function () {
        var copiedRand = replayable_random_1.alea.mutFromPlain(this.randState); // FIXME
        return new SimpleBlockFactory(this.replica, this.seq, copiedRand, new Map(this.generated));
    };
    SimpleBlockFactory.prototype.toJSON = function () {
        return {
            replica: this.replica,
            seq: this.seq,
            randState: this.randState,
            generated: Array.from(this.generated),
        };
    };
    // Status
    SimpleBlockFactory.prototype.isAppendable = function (l, length) {
        var gen = this.generated.get(l.nth());
        return (l.replica() === this.replica &&
            l.hasIntSucc(length) && // enough room for last pos
            gen !== undefined &&
            gen[0].upper() === l.offset());
    };
    SimpleBlockFactory.prototype.isPrependable = function (length, u) {
        var gen = this.generated.get(u.nth());
        return (u.replica() === this.replica &&
            u.hasIntPred(length) && // enough room for first pos
            gen !== undefined &&
            gen[0].lower === u.offset());
    };
    // Modification
    SimpleBlockFactory.prototype.register = function (l, length) {
        var registeredRange = u32_range_1.U32Range.fromLength(l.offset(), length);
        var gen = this.generated.get(l.nth());
        if (gen !== undefined) {
            var merged = void 0;
            if (gen[0].lower < registeredRange.lower) {
                merged = gen[0].append(registeredRange);
            }
            else {
                merged = registeredRange.append(gen[0]);
            }
            gen[0] = merged; // mutation
        }
        else {
            this.generated.set(l.nth(), [registeredRange, 0]);
        }
    };
    /** @override */
    SimpleBlockFactory.prototype.garbageCollectPos = function (l, length) {
        var nth = l.nth();
        var gen = this.generated.get(nth);
        if (l.replica() === this.replica && gen !== undefined) {
            gen[1] = gen[1] + length;
            if (gen[0].length === gen[1]) {
                this.generated.delete(nth);
                if (nth === this.seq) {
                    this.seq++; // prevent double generation of positions
                }
            }
        }
    };
    // Impl
    /** @override */
    SimpleBlockFactory.prototype.posBetween = function (l, length, u) {
        var _this = this;
        assert_1.heavyAssert(function () { return l.compare(u) === -1 /* BEFORE */; }, "l < u");
        assert_1.assert(function () { return number_1.isU32(length); }, "length ∈ u32");
        assert_1.assert(function () { return length > 0; }, "length is strictly positive");
        assert_1.assert(function () { return number_1.isU32(_this.seq + length); }, "no overflow");
        var result;
        if (this.isAppendable(l, length)) {
            result = l.intSucc(1);
        }
        else if (this.isPrependable(length, u)) {
            result = u.intPred(length);
        }
        else {
            var seqL = infiniteSequence(l.parts, simple_pos_part_1.SimplePosPart.BOTTOM);
            var seqU = infiniteSequence(u.parts, simple_pos_part_1.SimplePosPart.TOP);
            var parts = [];
            var partL = seqL.next().value;
            var partU = seqU.next().value;
            while (partU.priority - partL.priority < 2) {
                // Cannot insert a new part between partL and partU
                if (partL.replica === partU.replica) {
                    // Split
                    parts.push(partL);
                }
                else {
                    parts.push(partL.withOffset(number_1.U32_TOP)); // Let room for append
                }
                partL = seqL.next().value;
                partU = seqU.next().value;
            }
            var priority = replayable_random_1.distrib.mutU32Between(partL.priority + 1)(partU.priority)(this.randState);
            // priority ∈ ]partL.priority, partU.priority[
            // partL.priority exclusion ensures a dense set
            parts.push(simple_pos_part_1.SimplePosPart.from(priority, this.replica, this.seq, U32_MID));
            this.seq++; // a new block was generated
            result = simple_pos_1.SimplePos.from(parts);
        }
        this.register(result, length);
        return result;
    };
    return SimpleBlockFactory;
}(block_factory_1.BlockFactory));
exports.SimpleBlockFactory = SimpleBlockFactory;
//# sourceMappingURL=simple-block-factory.js.map