/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
import { hashCodeOf } from "../util/number";
/**
 * List which can only be remotely updated using operations.
 * An operation must be delivered exactly once.
 * An insertion must be delivered before a removal that depends on.
 */
var OpReplicatedList = /** @class */ (function () {
    function OpReplicatedList() {
    }
    /**
     * @param prefix
     * @return Concatenated version prefixed by `prefix`.
     */
    OpReplicatedList.prototype.concatenated = function (prefix) {
        return this.reduceBlock(function (acc, b) { return acc.concat(b.content); }, prefix);
    };
    /**
     * Non-cryptographic way to approximate object identity.
     * Do not take the blocks' content into account.
     */
    OpReplicatedList.prototype.structuralHashCode = function () {
        return this.reduceBlock(function (acc, b) { return hashCodeOf([acc, b.structuralHashCode()]); }, 0);
    };
    OpReplicatedList.prototype.applyOp = function (delta) {
        if (delta.isLengthBlock()) {
            return this.remove(delta);
        }
        else {
            return this.insert(delta);
        }
    };
    return OpReplicatedList;
}());
export { OpReplicatedList };
//# sourceMappingURL=op-replicated-list.js.map