/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
import { assert } from "./assert";
export var U32_BOTTOM = 0;
export var U32_TOP = 4294967295; // 2^32 - 1
/**
 * @param n
 * @return Is {@link n} an unsigned integer 32?
 */
export var isU32 = function (n) {
    return typeof n === "number" &&
        Number.isSafeInteger(n) &&
        U32_BOTTOM <= n &&
        n <= U32_TOP;
};
/**
 * @param n1
 * @param n2
 * @return Order relation between {@link n1} and {@link n2}.
 */
export function compareU32(n1, n2) {
    assert(function () { return isU32(n1); }, "n1 ∈ u32");
    assert(function () { return isU32(n2); }, "n2 ∈ u32");
    if (n1 < n2) {
        return -1 /* BEFORE */;
    }
    else if (n1 > n2) {
        return 1 /* AFTER */;
    }
    else {
        return 0 /* EQUAL */;
    }
}
/**
 * @param values
 * @return Non-cryptographic way to approximate identity of {@link values }.
 */
export var hashCodeOf = function (values) {
    return values.reduce(function (acc, v) { return (((acc * 17) >>> 0) + v) >>> 0; });
};
export var absoluteSubstraction = function (a, b) {
    return a < b ? b - a : a - b;
};
//# sourceMappingURL=number.js.map