import { Pos } from "./pos";
import { Ordering } from "../util/ordering";
import { FromPlain } from "../util/data-validation";
/**
 * An anchor can be used to model the cursor of a participant in a
 * repliacted list.
 */
export declare class Anchor<P extends Pos<P>> {
    /**
     * @example
     * Anchor(p, false) is just before p
     * Anchor(p, true) is just after p
     * There does not exist a position between Anchor(p) and p
     *
     * @param ref position to which this anchor is relative to
     * @param isAfter is the anchor after `ref`?
     */
    static from<P extends Pos<P>>(ref: P, isAfter: boolean): Anchor<P>;
    static fromPlain<P extends Pos<P>>(posFromPlain: FromPlain<P>): FromPlain<Anchor<P>>;
    /**
     * Position to which this anchor is relative to
     */
    readonly ref: P;
    /**
     * Is the anchor after {@link Anchor#ref}?
     */
    readonly isAfter: boolean;
    /**
     * @example
     * Anchor(p) is just before p
     * There not exists another position between Anchor(p) and p
     *
     * @param ref position to which this anchor is relative to
     * @param isAfter is the anchor after `ref`?
     */
    protected constructor(ref: P, isAfter: boolean);
    /**
     * Globally unique identifier of the author which generated this block.
     */
    replica(): ReturnType<P["replica"]>;
    /**
     * @param other
     * @return this [Order relation] other
     */
    compare(other: Anchor<P>): Ordering;
    /**
     * @example
     * Anchor(p).compareTo(q) == Ordering.Before if p <= q
     *
     * @param pos
     * @return Is before or after {@link pos}?
     */
    compareWith(pos: P): Ordering.BEFORE | Ordering.AFTER;
}
