import { u32 } from "../util/number";
/**
 * Concatenable and sliceable types.
 * Native types Array and String implements this interface.
 */
export interface Concat<E extends Concat<E>> {
    /**
     * Number of elements.
     */
    readonly length: u32;
    /**
     * "abc".slice(0, 3) === "abc"
     * "abc".slice(1, 2) === "b"
     *
     * @param lower index where the slice starts.
     * @param excludedUpper excluded index where the slice ends.
     * @return Slice between indexes [`lower', `excludedUpper'[.
     */
    readonly slice: (lower: u32, excludedUpper: u32) => E;
    /**
     * @param other appended elements
     * @return `other' appended to `this'.
     */
    readonly concat: (other: E) => E;
}
/**
 * Minimal implementation of Concatenable.
 * This enables to model a number of elements without their availability.
 *
 * For instance ConcatLength(3) can represent "abc", [1, 2, 3], or any
 * 3-elements list.
 */
export declare class ConcatLength implements Concat<ConcatLength> {
    readonly length: u32;
    /**
     * @param x candidate
     * @return object from `x', or undefined if `x' is not valid.
     */
    static fromPlain(x: unknown): ConcatLength | undefined;
    /** Nominal typing */
    private readonly brandConcatenableLength;
    /**
     * @param length {@link Concat#length}
     */
    constructor(length: u32);
    /** @override */
    slice(lower: u32, excludedUpper: u32): ConcatLength;
    /** @override */
    concat(other: ConcatLength): ConcatLength;
}
