import { MutRand } from "replayable-random";
import { FromPlain } from "../../util/data-validation";
import { SimpleDotPos } from "./simple-dot-pos";
import { u32 } from "../../util/number";
import { BlockFactory } from "../../core/block-factory";
import { Concat } from "../../core/concat";
import { Block } from "../../core/block";
/**
 * Factory of block with {@link SimplePos } as implementation of
 * {@link Pos}.
 * The strategy of generation is to genearte random positions between two
 * defined positions.
 */
export declare class SimpleDotBlockFactory extends BlockFactory<SimpleDotPos> {
    /**
     * @param posBounds {@link SimpleBlockFactory#replica }
     * @param seq {@link SimpleBlockFactory#seq }
     * @param randState random generator's state
     */
    protected constructor(replica: u32, seq: u32, randState: MutRand);
    /**
     * The random generator is seeded with both globalSeed and replica.
     *
     * @param replica {@link SimpleBlockFactory#replica }
     * @param globalSeed Seed for predictable random generation.
     * @return New factory with 0 as {@link SimpleBlockFactory#seq }
     */
    static from(replica: u32, globalSeed: string): SimpleDotBlockFactory;
    /**
     * @param x candidate
     * @return object from `x', or undefined if `x' is not valid.
     */
    static fromPlain(x: unknown): SimpleDotBlockFactory | undefined;
    /**
     * See {@link BlockFactoryConstructor#blockFromPlain}
     */
    static blockFromPlain<E extends Concat<E>>(g: FromPlain<E>): FromPlain<Block<SimpleDotPos, E>>;
    /** @Override */
    readonly replica: u32;
    /** @Override */
    seq: u32;
    randState: MutRand;
    /**
     * @return Deep copy of this.
     */
    copy(): SimpleDotBlockFactory;
    /**
     * Increase {@link SimpleBlockFactory#seq } by {@link by }
     *
     * @param by
     */
    increaseSeq(by: u32): void;
    /**
     * @param l
     * @param Can `length` be appended to `l`?
     */
    isAppendable(l: SimpleDotPos, length: u32): boolean;
    /** @override */
    posBetween(l: SimpleDotPos, length: u32, u: SimpleDotPos): SimpleDotPos;
}
