import { u32, i32 } from "../../util/number";
import { Block, LengthBlock } from "../../core/block";
import { Pos } from "../../core/pos";
import { Concat } from "../../core/concat";
import { U32Range } from "../../core/u32-range";
import { Del, Ins } from "../../core/local-operation";
import { BlockListContext } from "../../core/block-list-context";
import { BlockFactory } from "../../core/block-factory";
import { FromPlain } from "../../util/data-validation";
import { Anchor } from "../../core/anchor";
export declare type Node<P extends Pos<P>, E extends Concat<E>> = ValuedNode<P, E> | undefined;
export declare function lengthOf<P extends Pos<P>, E extends Concat<E>>(node: Node<P, E>): u32;
export declare class ValuedNode<P extends Pos<P>, E extends Concat<E>> extends BlockListContext<P, E> {
    static leaf<P extends Pos<P>, E extends Concat<E>>(block: Block<P, E>): ValuedNode<P, E>;
    /**
     * @param blockFromPlain
     * @return function that accepts a value and returns a Node from
     *  the value, or undefined if the value is mal-formed
     */
    static fromPlain<P extends Pos<P>, E extends Concat<E>>(blockFromPlain: FromPlain<Block<P, E>>): FromPlain<ValuedNode<P, E>>;
    readonly left: Node<P, E>;
    readonly block: Block<P, E>;
    protected isSelfRemoved: boolean;
    length: u32;
    rank: u32;
    readonly right: Node<P, E>;
    protected constructor(l: Node<P, E>, block: Block<P, E>, r: Node<P, E>);
    setLeft(l: Node<P, E>): void;
    replace(block: Block<P, E>): void;
    setRight(r: Node<P, E>): void;
    update(): void;
    balanceFactor(): i32;
    isBalanced(): boolean;
    isRightUnbalanced(): this is this & {
        right: ValuedNode<P, E>;
    };
    isRightOriented(): this is this & {
        right: ValuedNode<P, E>;
    };
    isLeftOriented(): this is this & {
        left: ValuedNode<P, E>;
    };
    isSlighlyLeftUnbalanced(): this is this & {
        left: ValuedNode<P, E>;
    };
    rotateLeft(this: this & {
        right: ValuedNode<P, E>;
    }): ValuedNode<P, E>;
    rotateRight(this: this & {
        left: ValuedNode<P, E>;
    }): ValuedNode<P, E>;
    balance(): ValuedNode<P, E>;
    updated(): Node<P, E>;
    reduceBlock<U>(f: (acc: U, b: Block<P, E>) => U, prefix: U): U;
    current(): Block<P, E>;
    currentRelativeIndex(): u32;
    min(): Block<P, E>;
    max(): Block<P, E>;
    predecessor(): Block<P, E> | undefined;
    successor(): Block<P, E> | undefined;
    insertMin(iBlock: Block<P, E>): ValuedNode<P, E>;
    insertMax(iBlock: Block<P, E>): ValuedNode<P, E>;
    /** @Override */
    indexFromLeft(anchor: Anchor<P>, minIndex: u32): u32;
    /** @Override */
    indexFromRight(anchor: Anchor<P>, minIndex: u32): u32;
    /** @Override */
    anchorAtLeft(relIndex: u32, isAfter: boolean, f: BlockFactory<P>): Anchor<P>;
    /** @Override */
    anchorAtRight(relIndex: u32, isAfter: boolean, f: BlockFactory<P>): Anchor<P>;
    /** @Override */
    insertableLeft(iBlock: Block<P, E>): Block<P, E>[];
    /** @Override */
    insertableRight(iBlock: Block<P, E>): Block<P, E>[];
    /** @Override */
    insertPredecessor(iBlock: Block<P, E>): void;
    /** @Override */
    insertSuccessor(iBlock: Block<P, E>): void;
    /** @Override */
    insertLeft(iBlock: Block<P, E>, minIndex: u32): Ins<E>[];
    /** @Override */
    insertRight(iBlock: Block<P, E>, minIndex: u32): Ins<E>[];
    /** @Override */
    insertAtLeft(idx: u32, items: E, f: BlockFactory<P>): Block<P, E>;
    /** @Override */
    insertAtRight(idx: u32, items: E, f: BlockFactory<P>): Block<P, E>;
    /** @Override */
    removeCurrent(): void;
    removeMin(): Node<P, E>;
    removeMax(): Node<P, E>;
    /** @Override */
    removePredecessor(): void;
    /** @Override */
    removeSuccessor(): void;
    /** @Override */
    removeLeft(dBlock: LengthBlock<P>, minIndex: u32): Del[];
    /** @Override */
    removeRight(dBlock: LengthBlock<P>, minIndex: u32): Del[];
    /** @Override */
    removeAtLeft(dRange: U32Range, minIndex: u32): LengthBlock<P>[];
    /** @Override */
    removeAtRight(dRange: U32Range, minIndex: u32): LengthBlock<P>[];
}
