import { Pos } from "../../core/pos";
import { Concat } from "../../core/concat";
import { OpReplicatedList, OpEditableReplicatedList } from "../../core/op-replicated-list";
import { OpAvlList, EditableOpAvlList } from "./op-avl-list";
import { BlockFactory, BlockFactoryConstructor } from "../../core/block-factory";
import { DeltaReplicatedList, DeltaEditableReplicatedList } from "../../core/delta-replicated-list";
import { DotPos } from "../../core/dot-pos";
import { FromPlain } from "../../util/data-validation";
/**
 * @return operation-based empty AVL list that cannot be locally modified
 */
export declare function opList<P extends Pos<P>, E extends Concat<E>>(): OpReplicatedList<P, E>;
/**
 * @param f
 * @param itemsFromPlain
 * @param x candidate
 * @return operation-based AVL list from `x`, or undefined if `x` is mal-formed
 */
export declare const opListFromPlain: typeof OpAvlList.fromPlain;
/**
 *  @param factory block factory
 * @param v empty value (used for type inference)
 * @return operation-based empty AVL list that can be locally modified
 */
export declare function opEditableList<P extends Pos<P>, E extends Concat<E>>(factory: BlockFactory<P>, v: E): OpEditableReplicatedList<P, E>;
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
export declare const opEditableListFromPlain: typeof EditableOpAvlList.fromPlain;
/**
 * @return delta-based empty AVL list that cannot be locally modified
 */
export declare function deltaList<P extends DotPos<P>, E extends Concat<E>>(): DeltaReplicatedList<P, E>;
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
export declare function deltaListFromPlain<P extends DotPos<P>, E extends Concat<E>>(f: BlockFactoryConstructor<P>, itemsFromPlain: FromPlain<E>): FromPlain<DeltaReplicatedList<P, E>>;
/**
 * @param factory block factory
 * @param v empty value (used for type inference)
 * @return delta-based empty AVL list that can be locally modified
 */
export declare function deltaEditableList<P extends DotPos<P>, E extends Concat<E>>(factory: BlockFactory<P>, v: E): DeltaEditableReplicatedList<P, E>;
/**
 * @param f
 * @param itemsFromPlain
 * @return function that accepts a value and attempt to build a list.
 *  It returns the built list if it succeeds, or undefined if it fails.
 */
export declare function deltaEditableListFromPlain<P extends DotPos<P>, E extends Concat<E>>(f: BlockFactoryConstructor<P>, itemsFromPlain: FromPlain<E>): FromPlain<DeltaEditableReplicatedList<P, E>>;
