import { Pos } from "../../core/pos";
import { Concat } from "../../core/concat";
import { OpReplicatedList, OpEditableReplicatedList } from "../../core/op-replicated-list";
import { Node } from "./avl-list-node";
import { u32 } from "../../util/number";
import { Block, LengthBlock } from "../../core/block";
import { Ins, Del } from "../../core/local-operation";
import { BlockFactory, BlockFactoryConstructor } from "../../core/block-factory";
import { FromPlain } from "../../util/data-validation";
import { Anchor } from "../../core/anchor";
/**
 * An {@see OpReplicatedList } that uses an AVL tree.
 */
export declare class OpAvlList<P extends Pos<P>, E extends Concat<E>> extends OpReplicatedList<P, E> {
    /**
     * @param v empty value of type E
     * @return new empty list.
     */
    static empty<P extends Pos<P>, E extends Concat<E>>(): OpAvlList<P, E>;
    /**
     * @param f
     * @param itemsFromPlain
     * @return function that accepts a value and attempt to build a list.
     *  It returns the built list if it succeeds, or undefined if it fails.
     */
    static fromPlain<P extends Pos<P>, E extends Concat<E>>(f: BlockFactoryConstructor<P>, itemsFromPlain: FromPlain<E>): FromPlain<OpReplicatedList<P, E>>;
    /**
     * AVL root.
     */
    protected root: Node<P, E>;
    /**
     * @param root {@see EditableOpAvlList#root }
     */
    protected constructor(root: Node<P, E>);
    /** @Override */
    readonly length: u32;
    /** @Override */
    indexFrom(anchor: Anchor<P>): u32;
    /** @Override */
    reduceBlock<U>(f: (acc: U, b: Block<P, E>) => U, prefix: U): U;
    /** @Override */
    insertable(delta: Block<P, E>): Block<P, E>[];
    /** @Override */
    insert(delta: Block<P, E>): Ins<E>[];
    /** @Override */
    remove(delta: LengthBlock<P>): Del[];
}
/**
 * An {@see EditableOpReplicatedList } that uses an AVL tree.
 */
export declare class EditableOpAvlList<P extends Pos<P>, E extends Concat<E>> extends OpAvlList<P, E> implements OpEditableReplicatedList<P, E> {
    /**
     * @param factory factory of blocks
     * @param v empty value of type E (only used for type inference)
     * @return new empty list.
     */
    static emptyWith<P extends Pos<P>, E extends Concat<E>>(factory: BlockFactory<P>): EditableOpAvlList<P, E>;
    /**
     *
     * @param f
     * @param itemsFromPlain
     * @return function that accepts a value and attempt to build a list.
     *  It returns the built list if it succeeds, or undefined if it fails.
     */
    static fromPlain<P extends Pos<P>, E extends Concat<E>>(f: BlockFactoryConstructor<P>, itemsFromPlain: FromPlain<E>): FromPlain<OpEditableReplicatedList<P, E>>;
    /**
     * factory of blocks.
     */
    protected readonly factory: BlockFactory<P>;
    /**
     * @param root {@see EditableOpAvlList#root }
     * @param factory {@see EditableOpAvlList#factory }
     */
    protected constructor(root: Node<P, E>, factory: BlockFactory<P>);
    /** @Override */
    anchorAt(index: u32, isAfter: boolean): Anchor<P>;
    /** @Override */
    remove(delta: LengthBlock<P>): Del[];
    /** @Override */
    insertAt(index: u32, items: E): Block<P, E>;
    /** @Override */
    removeAt(index: u32, length: u32): LengthBlock<P>[];
}
