import { Block, LengthBlock } from "../../core/block";
import { BlockFactory, BlockFactoryConstructor } from "../../core/block-factory";
import { Concat } from "../../core/concat";
import { Pos } from "../../core/pos";
import { Ins, Del } from "../../core/local-operation";
import { u32 } from "../../util/number";
import { Sentinel } from "./linked-list-cell";
import { OpReplicatedList, OpEditableReplicatedList } from "../../core/op-replicated-list";
import { FromPlain } from "../../util/data-validation";
import { Anchor } from "../../core/anchor";
/**
 * An {@see OpReplicatedList } that uses an AVL tree.
 */
export declare class OpLinkedList<P extends Pos<P>, E extends Concat<E>> extends OpReplicatedList<P, E> {
    /**
     * @param v empty value of type E
     * @return new empty list.
     */
    static empty<P extends Pos<P>, E extends Concat<E>>(): OpLinkedList<P, E>;
    /**
     * @param f
     * @param itemsFromPlain
     * @return function that accepts a value and attempt to build a list.
     *  It returns the built list if it succeeds, or undefined if it fails.
     */
    static fromPlain<P extends Pos<P>, E extends Concat<E>>(f: BlockFactoryConstructor<P>, itemsFromPlain: FromPlain<E>): FromPlain<OpReplicatedList<P, E>>;
    /**
     * Chain entry.
     */
    protected readonly root: Sentinel<P, E>;
    /** @Override */
    length: u32;
    /**
     * New empty list.
     */
    protected constructor(root: Sentinel<P, E>, length: u32);
    /** @Override */
    reduceBlock<U>(f: (acc: U, b: Block<P, E>) => U, prefix: U): U;
    /** @Override */
    indexFrom(anchor: Anchor<P>): u32;
    /** @Override */
    insertable(delta: Block<P, E>): Block<P, E>[];
    /** @Override */
    insert(block: Block<P, E>): Ins<E>[];
    /** @Override */
    remove(block: LengthBlock<P>): Del[];
}
/**
 * An {@see EditableOpReplicatedList } that uses an AVL tree.
 */
export declare class EditableOpLinkedList<P extends Pos<P>, E extends Concat<E>> extends OpLinkedList<P, E> implements OpEditableReplicatedList<P, E> {
    /**
     * @param f
     * @param itemsFromPlain
     * @return function that accepts a value and attempt to build a list.
     *  It returns the built list if it succeeds, or undefined if it fails.
     */
    static fromPlain<P extends Pos<P>, E extends Concat<E>>(f: BlockFactoryConstructor<P>, itemsFromPlain: FromPlain<E>): FromPlain<OpEditableReplicatedList<P, E>>;
    /**
     * @param factory factory of blocks
     * @param v empty value of type E (only used for type inference)
     * @return new empty list.
     */
    static emptyWith<P extends Pos<P>, E extends Concat<E>>(factory: BlockFactory<P>): EditableOpLinkedList<P, E>;
    /**
     * factory of blocks.
     */
    protected readonly factory: BlockFactory<P>;
    /**
     * @param factory strategy of block generation.
     */
    protected constructor(root: Sentinel<P, E>, length: u32, factory: BlockFactory<P>);
    /** @Override */
    anchorAt(index: u32, isAfter: boolean): Anchor<P>;
    /** @Override */
    remove(delta: LengthBlock<P>): Del[];
    /** @Override */
    insertAt(index: u32, items: E): Block<P, E>;
    /** @Override */
    removeAt(index: u32, length: u32): LengthBlock<P>[];
}
