import { MutRand } from "replayable-random";
import { FromPlain } from "../../util/data-validation";
import { SimplePos } from "./simple-pos";
import { u32 } from "../../util/number";
import { BlockFactory } from "../../core/block-factory";
import { Concat } from "../../core/concat";
import { Block } from "../../core/block";
import { U32Range } from "../../core/u32-range";
export interface SimpleBlockFactoryJSON {
    readonly replica: u32;
    readonly seq: u32;
    readonly randState: MutRand;
    readonly generated: ReadonlyArray<readonly [u32, [U32Range, u32]]>;
}
/**
 * Factory of block with {@link SimplePos } as implementation of
 * {@link Pos}.
 * The strategy of generation is to genearte random positions between two
 * defined positions.
 */
export declare class SimpleBlockFactory extends BlockFactory<SimplePos> {
    /**
     * The random generator is seeded with both globalSee>eplica.
     *
     * @param replica {@link SimpleBlockFactory#replica }>
     * @param globalSeed Seed for predictable random generation.
     * @return New factory with 0 as {@link SimpleBlockFactory#seq }
     */
    static from(replica: u32, globalSeed: string): SimpleBlockFactory;
    /**
     * @param x candidate
     * @return object from `x', or undefined if `x' is not valid.
     */
    static fromPlain(x: unknown): SimpleBlockFactory | undefined;
    /**
     * See {@link BlockFactoryConstructor#blockFromPlain}
     */
    static blockFromPlain<E extends Concat<E>>(g: FromPlain<E>): FromPlain<Block<SimplePos, E>>;
    /** @Override */
    readonly replica: u32;
    /** @Override */
    seq: u32;
    randState: MutRand;
    /**
     * Positions already generated.
     * This is kept to prevent double generation of the same positions.
     */
    readonly generated: Map<u32, [U32Range, u32]>;
    /**
     * @param posBounds {@link SimpleBlockFactory#replica }
     * @param seq {@link SimpleBlockFactory#seq }
     * @param randState random generator's state
     */
    protected constructor(replica: u32, seq: u32, randState: MutRand, generated: Map<u32, [U32Range, u32]>);
    /**
     * @return Deep copy of this.
     */
    copy(): SimpleBlockFactory;
    toJSON(): SimpleBlockFactoryJSON;
    isAppendable(l: SimplePos, length: u32): boolean;
    isPrependable(length: u32, u: SimplePos): boolean;
    register(l: SimplePos, length: u32): void;
    /** @override */
    garbageCollectPos(l: SimplePos, length: u32): void;
    /** @override */
    posBetween(l: SimplePos, length: u32, u: SimplePos): SimplePos;
}
