/** @internal */
export declare type NonFunctionNames<T> = {
    [k in keyof T]: T[k] extends Function ? never : k;
}[keyof T];
/** @internal */
export declare type Unknown<T> = {
    [k in NonFunctionNames<T>]?: unknown;
};
/**
 * Example:
 * Given `x: unknown`
 * `isObject<{ p: number }>(x) && typeof x.p === "number"`
 * enables to test if x is conforms to `{ p: number }`.
 *
 * @internal
 * @param x
 * @param Is `x' a non-null object?
 */
export declare const isObject: <T>(x: unknown) => x is Unknown<T>;
export declare type FromPlain<T> = (x: unknown) => T | undefined;
export declare function fromArray<T>(x: unknown[], f: FromPlain<T>): T[] | undefined;
export declare function or<A, B>(f: FromPlain<A>, g: FromPlain<B>): FromPlain<A | B>;
